@@ -92,4 +92,10 @@ DEFAULT_LOCAL_SEARCH_RADIUS=10000  # in meters
 # API access
 GOOGLE_API_KEY = 'ABQIAAAAv84YYgTIjdezewgb8xl5_xTKlax5G-CAZlpGqFgXfh-jq3S0yRS6XLrXE9CkHPS6KDCig4gHvHK3lw'
 GOOGLE_PLACES_API_KEY = 'AIzaSyAUbkZEu97uzwbl9IA4DNEMLlPg_gPlNTw'
-GEONAMES_USERNAME = "modrana"
\ No newline at end of file
+GEONAMES_USERNAME = "modrana"
+
+# tile download status codes
+TILE_DOWNLOAD_SUCCESS = 0
+TILE_DOWNLOAD_ERROR = 1
+TILE_DOWNLOAD_TEMPORARY_ERROR = 2
+TILE_DOWNLOAD_QUEUE_FULL = 3
